# coding: utf-8
"""JSON type"""

# standard
import json
from collections import OrderedDict

# third
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "JSON"
FILETYPE_EXAMPLE_LOADER = """{
            'general': {
                'user': 'admin'
            },
            'keys': [
                {'key': 'value1'},
                {'key': 'value2'},
                'alone'
            ]
        }
"""
FILETYPE_EXAMPLE_DUMPER = """{
            "general": {
                "user": "admin"
            },
            "keys": [
                {"key": "value1"},
                {"key": "value2"},
                "alone"
            ]
        }
"""


def loader(data_string):
    return json.loads(data_string)


def dumper(data):
    return json.dumps(data, indent=4)


def colorizer(content):
    return highlight(
        content, lexers.JsonLexer(), formatters.TerminalFormatter()
    )



# coding: utf-8
"""INI type"""

# standard
import io
import sys
import configparser
from collections import OrderedDict

# third
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "INI"
FILETYPE_EXAMPLE_LOADER = """OrderedDict([
            ('general', OrderedDict([
                ('user', 'admin')
            ])),
            ('keys', OrderedDict([
                ('key1', 'value1'),
                ('key2', 'value2')
            ]))
        ])
"""
FILETYPE_EXAMPLE_DUMPER = """[general]
        user = admin
        [keys]
        key1 = value1
        key2 = value2
"""


def loader(data_string):
    config = configparser.ConfigParser()
    config.read_string(data_string)
    return config._sections


def dumper(data):
    parser = configparser.ConfigParser()
    parser.read_dict(data)

    buf = io.StringIO()
    parser.write(buf)

    return buf.getvalue()


def colorizer(content):
    return highlight(content, lexers.IniLexer(), formatters.TerminalFormatter())



# coding: utf-8
"""YAML type"""

# standard
from collections import OrderedDict

# third
import yaml
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "YAML"
FILETYPE_EXAMPLE_LOADER = """{
            'general': {
                'user': 'admin'
            },
            'keys': [
                {'key': 'value1'},
                {'key': 'value2'},
                'alone'
            ]
        }
"""
FILETYPE_EXAMPLE_DUMPER = """general:
          user: "admin"
        keys:
          - key1: value1
          - key2: value2
          - alone
"""


def loader(data_string):
    return yaml.safe_load(data_string)


def dumper(data):
    # Dump YAML thanks to https://github.com/dale3h/python-lovelace.
    def ordered_dump(content, stream=None, Dumper=yaml.Dumper, **kwargs):
        """YAML dumper for OrderedDict."""

        class OrderedDumper(Dumper):
            """Wrapper class for YAML dumper."""

            def ignore_aliases(self, content):
                """Disable aliases in YAML dump."""
                return True

            def increase_indent(self, flow=False, indentless=False):
                """Increase indent on YAML lists."""
                return super(OrderedDumper, self).increase_indent(flow, False)

        def _dict_representer(dumper, content):
            """Function to represent OrderDict and derivitives."""
            return dumper.represent_mapping(
                yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG, content.items()
            )

        OrderedDumper.add_multi_representer(OrderedDict, _dict_representer)

        OrderedDumper.add_representer(
            str, yaml.representer.SafeRepresenter.represent_str
        )

        return yaml.dump(content, stream, OrderedDumper, **kwargs)

    return ordered_dump(
        content=data, Dumper=yaml.SafeDumper, default_flow_style=False
    )


def colorizer(content):
    return highlight(
        content, lexers.YamlLexer(), formatters.TerminalFormatter()
    )



# coding: utf-8
"""TOML type"""

# standard
from collections import OrderedDict

# third
import toml
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "TOML"
FILETYPE_EXAMPLE_LOADER = """{
            'general': {
                'user': 'admin'
            },
            'keys': [
                {'key': 'value1'},
                {'key': 'value2'}
            ]
        }
"""
FILETYPE_EXAMPLE_DUMPER = """# keys = [ "alone" ] # produce an error on dump only
        [[keys]]
        key = "value1"
        [[keys]]
        key = "value2"
        [general]
        user = "admin"
"""


def loader(data_string):
    return toml.loads(data_string)


def dumper(data):
    return toml.dumps(data)


def colorizer(content):
    return highlight(
        content, lexers.TOMLLexer(), formatters.TerminalFormatter()
    )



# coding: utf-8
from wildq.filetypes import hcl_type
from wildq.filetypes import ini_type
from wildq.filetypes import json_type
from wildq.filetypes import toml_type
from wildq.filetypes import xml_type
from wildq.filetypes import yaml_type

# Auto Doc
for ft in [hcl_type, ini_type, json_type, toml_type, xml_type, yaml_type]:
    ft.loader.__doc__ = f"""Pass data as string to {ft.FILETYPE_NAME} loader.

    Args:
        data_string (str): String content to load, usually a file content.

    Returns:
        Loaded content. Return type depends on content format,
        should be dict, list, string etc.

    Examples:
        >>> from wildq.filetypes import {ft.FILETYPE_NAME.lower()}_type
        >>> with open("examples/{ft.FILETYPE_NAME.lower()}.{ft.FILETYPE_NAME.lower()}", "r") as fd:
        ...   data_string = fd.read()
        >>> data = {ft.FILETYPE_NAME.lower()}_type.loader(data_string)
        {ft.FILETYPE_EXAMPLE_LOADER}
    """

    ft.dumper.__doc__ = f"""Pass loaded data to {ft.FILETYPE_NAME} dumper.

    Args:
        data: Should be dict, list, string etc.

    Returns:
        str: Dumped data.

    Examples:
        >>> from wildq.filetypes import {ft.FILETYPE_NAME.lower()}_type
        >>> data = {ft.FILETYPE_NAME.lower()}_type.dumper(data_string)
        {ft.FILETYPE_EXAMPLE_DUMPER}
    """

    ft.colorizer.__doc__ = f"""Pass dumped data to {ft.FILETYPE_NAME} colorize.

    Args:
        content (str): string representation of data, usually a dumped data. 

    Returns:
        str: Colorized data.

    Examples:
        >>> from wildq.filetypes import {ft.FILETYPE_NAME.lower()}_type
        >>> data = {ft.FILETYPE_NAME.lower()}_type.colorizer(data_string)
        ...
    """



# standard
import sys

# third
import jq
import click

# local
import wildq.filetypes
from wildq._wildq_version import __version__


class Config:
    pass


# could be replaced by importlib / getattr but explicit is better than implicit
SUPPORTED_FILETYPES = {
    "hcl": wildq.filetypes.hcl_type,
    "ini": wildq.filetypes.ini_type,
    "json": wildq.filetypes.json_type,
    "toml": wildq.filetypes.toml_type,
    "xml": wildq.filetypes.xml_type,
    "yaml": wildq.filetypes.yaml_type,
}

PASS_CONFIG = click.make_pass_decorator(Config, ensure=True)


def compiler(jq_filter, data):
    return jq.compile(jq_filter).input(data)


@click.command()
# @click.option('--input', help='compose file to work with', type=click.File('r'), default=sys.stdin)
# @click.option('--output', help='compose file to work with', type=click.File('r'), default=sys.stdin)
@click.option(
    "-c",
    "--compact-output",
    is_flag=True,
    default=False,
    help="compact instead of pretty-printed output",
)
@click.option(
    "-r",
    "--raw",
    is_flag=True,
    default=False,
    help="output raw strings, not content texts",
)
@click.option(
    "-C",
    "--color-output",
    is_flag=True,
    default=False,
    help="colorize content (default), mutally exclusive with --monochrome-output",
)
@click.option(
    "-M",
    "--monochrome-output",
    is_flag=True,
    default=False,
    help="monochrome (don't colorize content), mutally exclusive with --color-output",
)
# @click.option('-S', '--sort', is_flag=True, default=False, help="sort keys of objects on output")
@click.option(
    "--hcl",
    is_flag=True,
    default=False,
    help="Combine --input hcl --output json, mutally exclusive with other Combined options",
)
@click.option(
    "--ini",
    is_flag=True,
    default=False,
    help="Combine --input ini --output json, mutally exclusive with other Combined options",
)
@click.option(
    "--json",
    is_flag=True,
    default=False,
    help="Combine --input json --output json, mutally exclusive with other Combined options",
)
@click.option(
    "--toml",
    is_flag=True,
    default=False,
    help="Combine --input toml --output json, mutally exclusive with other Combined options",
)
@click.option(
    "--xml",
    is_flag=True,
    default=False,
    help="Combine --input xml --output json, mutally exclusive with other Combined options",
)
@click.option(
    "--yaml",
    is_flag=True,
    default=False,
    help="Combine --input yaml --output json, mutally exclusive with other Combined options",
)
@click.option(
    "-i",
    "--input",
    type=click.Choice(list(SUPPORTED_FILETYPES.keys())),
    help="Define the content type of file, mutally exclusive with Combined option",
)
@click.option(
    "-o",
    "--output",
    type=click.Choice(list(SUPPORTED_FILETYPES.keys())),
    default=None,
    show_default=False,
    help="Define the content type of printed output, mutally exclusive with Combined option (default input format)",
)
@click.argument("jq_filter")
@click.argument("file", type=click.File("r"), default=sys.stdin)
@click.version_option(version=__version__)
def cli(*args, **kwargs):
    from pprint import pprint

    if (kwargs["input"] or kwargs["output"]) and any(
        [True for x in list(SUPPORTED_FILETYPES.keys()) if kwargs[x]]
    ):
        print(
            "When using combined option you can't use --input or --output, please use them explictly"
        )
        sys.exit(1)

    if kwargs["color_output"] and kwargs["monochrome_output"]:
        print(
            "--color-output and --monochrome-output are mutally exclusive, select one"
        )
        sys.exit(1)

    colorized = False
    if not kwargs["color_output"] and not kwargs["monochrome_output"]:
        colorized = True

    if kwargs["color_output"]:
        colorized = True

    if kwargs["monochrome_output"]:
        colorized = False

    input_fmt = None
    output_fmt = None

    raw = kwargs["raw"]

    # backward compatibility, if a type is specified without -i, output to json
    # enforce colorize to false and raw to true like first release of wq
    for spt in list(SUPPORTED_FILETYPES.keys()):
        if kwargs[spt]:
            input_fmt = spt
            output_fmt = "json"
            colorized = False
            raw = True

    # user specified input otherwise raise
    if not input_fmt:
        input_fmt = kwargs["input"]

    # if user specify input but not output, use default input format
    if not output_fmt:
        if kwargs.get("output") is None:
            output_fmt = input_fmt
        else:
            output_fmt = kwargs["output"]

    if not input_fmt or not output_fmt:
        if input_fmt is None:
            print("input option is empty")
        if output_fmt is None:
            print("output option is empty")
        print(
            "Issue when resolving input and output, getting confiuration %s",
            str(kwargs),
        )
        sys.exit(1)

    module_in = SUPPORTED_FILETYPES[input_fmt]
    module_out = SUPPORTED_FILETYPES[output_fmt]

    content = module_in.loader(kwargs["file"].read())

    # skip silently if content is empty
    if not content:
        return 0

    compiled = compiler(kwargs["jq_filter"], content)

    for line in compiled:
        if not isinstance(line, dict) and not isinstance(line, list):
            if raw:
                print(line.strip('"'))
            else:
                print(line)
            continue

        dumped = module_out.dumper(line)

        if colorized:
            print(module_out.colorizer(dumped))
        else:
            print(dumped)



# coding: utf-8
"""XML type"""

import json
from collections import OrderedDict

import xmltodict
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "XML"
FILETYPE_EXAMPLE_LOADER = """OrderedDict([
            ('root', OrderedDict([
                ('general', OrderedDict([
                    ('user', 'admin')
                ])),
                ('keys', OrderedDict([
                    ('element', [
                        OrderedDict([('key', 'value1')]),
                        OrderedDict([('key', 'value2')]),
                        'alone'
                    ])
                ]))
            ]))
        ])
"""
FILETYPE_EXAMPLE_DUMPER = """<?xml version="1.0" encoding="UTF-8"?>
        <root>
           <general>
              <user>admin</user>
           </general>
           <keys>
              <element>
                 <key>value1</key>
              </element>
              <element>
                 <key>value2</key>
              </element>
              <element>alone</element>
           </keys>
        </root>
"""


# local


def loader(data_string):
    return xmltodict.parse(data_string)


def dumper(data):
    if isinstance(data, dict) and len(data.keys()) == 1:
        return xmltodict.unparse(data, pretty=True, indent="   ")
    else:
        return xmltodict.unparse(
            {"virtualroot": data}, pretty=True, indent="   "
        )


def colorizer(content):
    return highlight(content, lexers.XmlLexer(), formatters.TerminalFormatter())



# coding: utf-8
"""HCL type"""

# standard
import json
from collections import OrderedDict

# third
import hcl
from pygments import highlight, lexers, formatters

FILETYPE_NAME = "HCL"
FILETYPE_EXAMPLE_LOADER = """{
            'general': {
                'user': 'admin'
            },
            'keys': [
                {'key': 'value1'},
                {'key': 'value2'}
            ]
        }
"""
FILETYPE_EXAMPLE_DUMPER = """general {
            user = "admin"
        }
        keys {
            key = "value1"
        }
        keys {
            key = "value2"
        }
"""


def loader(data_string):
    return hcl.loads(data_string)


def dumper(data):
    return json.dumps(data, indent=4)


def colorizer(content):
    # dump to hcl is not supported, so fallback to json
    return highlight(
        content,
        lexers.JsonLexer(),  # lexers.TerraformLexer()
        formatters.TerminalFormatter(),
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wildq', 'wildq.filetypes']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0',
 'Pygments>=2.7.2,<3.0.0',
 'click>=7.1.2,<8.0.0',
 'jq>=1.1.1,<2.0.0',
 'pyhcl>=0.4.4,<0.5.0',
 'toml>=0.10.1,<0.11.0',
 'xmltodict>=0.12.0,<0.13.0']

entry_points = \
{'console_scripts': ['wildq = wildq.__main__:main', 'wq = wildq.__main__:main']}

setup_kwargs = {
    'name': 'wildq',
    'version': '1.1.8',
    'description': 'Command-line TOML/JSON/INI/YAML/XML processor using jq c bindings.',
    'long_description': '\n\n# wildq - Command-line TOML/JSON/INI/YAML/XML/HCL processor using jq c bindings\n\n![GitHub](https://img.shields.io/github/license/ahmet2mir/wildq.svg)\n[![image](https://img.shields.io/pypi/pyversions/wildq.svg)](https://python.org/pypi/wildq)\n[![Build Status](https://github.com/ahmet2mir/wildq/workflows/Unit%20Tests/badge.svg)](https://github.com/ahmet2mir/wildq/actions)\n\nPurpose of this package is to provide a simple wrapper arround jq for different formats.\nI\'m tired of searching a package doing yaml jq, toml jq, ini jq etc. mainly used for scripting.\n\nThis script uses:\n* @mwilliamson [Python bindings](https://github.com/mwilliamson/jq.py) on top of @stedolan famous [jq](https://github.com/stedolan/jq/) lib\n* swiss knife for coloration [pygments](https://github.com/pygments/pygments)\n* binary built with [pyinstaller](https://github.com/pyinstaller/pyinstaller)\n* easy CLI with [click](https://github.com/pallets/click)\n* for supported types sources, check table `Supported file types`\n\n# Installation\n\nYou could install via pipx, pip, brew, deb, rpm or fallback to binary version.\n\n## Pipx\n\n`pipx` install and run python applications in isolated environments\n\n```sh\npipx install wildq\n```\n\n## Pip\n\n```sh\npip install wildq\n```\n\n## MacOS\n\n```\nbrew install ahmet2mir/tap/wildq\nbrew install ahmet2mir/tap/wq\n```\n\n## Debian (no gpg signature) >= 10\n\n```\nVERSION=$(curl -s "https://api.github.com/repos/ahmet2mir/wildq/releases/latest" | grep \'"tag_name":\' | sed -E \'s/.*"v([^"]+)".*/\\1/\')\ncurl -sL https://github.com/ahmet2mir/wildq/releases/download/v${VERSION}/wildq_${VERSION}-1_amd64.deb -o wildq_${VERSION}-1_amd64.deb\nsudo dpkg -i wildq_${VERSION}-1_amd64.deb\n\nwq --help\nwildq --help\n```\n\n## Centos (no gpg singature) >= 7\n\n```\nVERSION=$(curl -s "https://api.github.com/repos/ahmet2mir/wildq/releases/latest" | grep \'"tag_name":\' | sed -E \'s/.*"v([^"]+)".*/\\1/\')\ncurl -sL https://github.com/ahmet2mir/wildq/releases/download/v${VERSION}/wildq-${VERSION}-1.x86_64.rpm -o wildq-${VERSION}-1.x86_64.rpm\nsudo yum install -y ./wildq-${VERSION}-1.x86_64.rpm\n\nwq --help\nwildq --help\n```\n\n## GNU/Linux Binary\n\nCompiled using glibc 2.17, it should work on lot of stable/LTS distros.\n\n```\nmkdir -p ~/bin/\nVERSION=$(curl -s "https://api.github.com/repos/ahmet2mir/wildq/releases/latest" | grep \'"tag_name":\' | sed -E \'s/.*"v([^"]+)".*/\\1/\')\ncurl -sL https://github.com/ahmet2mir/wildq/releases/download/v${VERSION}/wildq-${VERSION}-linux-x86_64.tar.gz -o wildq-${VERSION}-linux-x86_64.tar.gz\ntar xvfz wildq-${VERSION}-linux-x86_64.tar.gz -C ~/bin\nexport PATH="~/bin:$PATH"\n\nwq --help\nwildq --help\n```\n\n## Windows\n\nWildq use [jq.py](https://github.com/mwilliamson/jq.py) and it\'s not yet available on windows platforms.\nI tried to compile it without windows machine and I failed, and I don\'t had the time to try to understand how Windows / C binding / Python works.\nIf anybody would contribute, there is an [open issue](https://github.com/mwilliamson/jq.py/issues/20) (jq and onigurama are \'compilable\' on Windows so I think that someone confortable with that OS could make it);\n\n\n# Supported file types\n\n| type | color | ordering | output | source                                                              |\n|------|-------|----------|--------|---------------------------------------------------------------------|\n| hcl  |  json |    no    |  json  | [pyhcl](https://github.com/virtuald/pyhcl) by @virtuald             |\n| ini  |  yes  |    no    |  yes   | [ConfigParser](https://docs.python.org/3/library/configparser.html) |\n| json |  yes  |    yes   |  yes   | [json](https://docs.python.org/3/library/json.html)                 |\n| toml |  yes  |    no    |  yes   | [toml](https://github.com/uiri/toml) by @uiri                       |\n| xml  |  yes  |    no    |  yes   | [xmldict](https://github.com/martinblech/xmltodict) by @martinblech |\n| yaml |  yes  |    yes   |  yes   | [pyyaml](https://github.com/yaml/pyyaml)                            |\n\n# Usage\n\n```\n$ wildq --help\nUsage: wildq [OPTIONS] JQ_FILTER [FILE]\n\nOptions:\n  -c, --compact-output            compact instead of pretty-printed output\n  -r, --raw                       output raw strings, not content texts\n  -C, --color-output              colorize content (default), mutally\n                                  exclusive with --monochrome-output\n\n  -M, --monochrome-output         monochrome (don\'t colorize content), mutally\n                                  exclusive with --color-output\n\n  --hcl                           Combine --input hcl --output json, mutally\n                                  exclusive with other Combined options\n\n  --ini                           Combine --input ini --output json, mutally\n                                  exclusive with other Combined options\n\n  --json                          Combine --input json --output json, mutally\n                                  exclusive with other Combined options\n\n  --toml                          Combine --input toml --output json, mutally\n                                  exclusive with other Combined options\n\n  --xml                           Combine --input xml --output json, mutally\n                                  exclusive with other Combined options\n\n  --yaml                          Combine --input yaml --output json, mutally\n                                  exclusive with other Combined options\n\n  -i, --input [hcl|ini|json|toml|xml|yaml]\n                                  Define the content type of file, mutally\n                                  exclusive with Combined option\n\n  -o, --output [hcl|ini|json|toml|xml|yaml]\n                                  Define the content type of printed output,\n                                  mutally exclusive with Combined option\n                                  (default input format)\n\n  --version                       Show the version and exit.\n  --help                          Show this message and exit.\n```\n\n> For backward compatibility in previous version only `--[yaml|json|toml|ini|xml|hcl]` was possible with default to json output.\nWe still keep Monochrome, raw and json output with thoses options.\nOutput was similar to `jq -MCr` (no color, no compact and no quote on single value)\n\nBut now, by default it\'s colorized, not raw and if you specify input using `-i` or `--input` output will be the same format.\n\nThere is also a shorter command `wq` comming with the package.\n\nLike `jq cli`, wildq supports both of stdin and file to the function\n\nSee examples to get some example.\n\nContent of `examples/json.json`\n\n```\n{\n    "general": {\n        "user": "admin"\n    },\n    "keys": [\n        {"key": "value1"},\n        {"key": "value2"},\n        "alone"\n    ]\n}\n```\n\n```sh\ncat examples/json.json | wildq -i json ".keys[]"\n{\n    "key": "value1"\n}\n{\n    "key": "value2"\n}\nalone\n```\n\nor\n\n```sh\nwildq -i json ".keys[]" examples/json.json\n{\n    "key": "value1"\n}\n{\n    "key": "value2"\n}\nalone\n```\nor\n\n```sh\nwq -i json ".keys[]" examples/json.json\n{\n    "key": "value1"\n}\n{\n    "key": "value2"\n}\nalone\n```\n\nFor TOML\n```sh\ncat examples/toml.toml | wildq -i toml ".keys[]"\n{\n    "key": "value1"\n}\n{\n    "key": "value2"\n}\nalone\n```\n\nFor INI (no array)\n```sh\ncat examples/ini.ini | wildq -i ini ".keys"\n{\n    "key1": "value1",\n    "key2": "value2"\n}\n```\n\nFor XML\n```sh\ncat examples/xml.xml | wildq -i xml "."\n{\n    "root": {\n        "general": {\n            "user": "admin"\n        },\n        "keys": {\n            "element": [\n                {\n                    "key": "value1"\n                },\n                {\n                    "key": "value2"\n                },\n                "alone"\n            ]\n        }\n    }\n}\n```\n\nFor YAML\n```sh\ncat examples/yaml.yaml  | wildq -i yaml ".keys[]"\n{\n    "key1": "value1"\n}\n{\n    "key2": "value2"\n}\nalone\n```\n\nFor HCL\n```sh\ncat examples/hcl.hcl  | wildq -i hcl ".keys[]"\n{\n    "key": "value1"\n}\n{\n    "key": "value2"\n}\n```\n\n## Tips and tricks\n\nLoop on keys in bash without creating a subshell\n\n```sh\nwildq -i toml "keys[]" examples/toml.toml | while read -r key \ndo\n    echo "Getting key ${key}"\ndone\n```\n\n## TODO\n\n- [x] add tests...\n- [x] add more control over filters and files\n- [x] use click for the CLI\n- [x] support different output\n- [ ] detect automagically filetype\n- [ ] support all jq types\n- [ ] ordering\n\n## Contributing\n\nMerge requests are welcome :)\n\n\n## License\n\nLicensed under the terms of the [Apache License, Version 2.0](http://www.apache.org/licenses/LICENSE-2.0).\n\n\n## Repository URL\n\nhttps://github.com/ahmet2mir/wildq\n',
    'author': 'Ahmet Demir',
    'author_email': 'me@ahmet2mir.eu',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ahmet2mir/wildq',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
